module Word2DictMatcher exposing (result)
{-|
@docs result
-}
import Dict exposing (..)
import Html exposing (..)



type alias Sentence =
    String

type alias Tokens =
    List String

type alias Histogram =
    Dict String Int

-- define your data (sentences) here, which you want to match the search string agains.
data =
    [ " "," " ]
    
-- define your search word, or, words!
searchString =
    [ " " ]
       
--this will count all words in the sentences for the dictionary values.
wordCount : String -> Dict String Int
wordCount sentence =
    sentence
        |> String.toLower
        |> String.words
        |> toHistogram
                           
wordsDict: List String -> Dict String Int
wordsDict vocabulary = 
    vocabulary
        |> List.concatMap String.words
        |> toHistogram   
        
searchDict =
    wordsDict searchString
        
dataDict =
    data |> sentenceHistograms
{-| Finds the sentences with the most occurances from a search string
Returns all the words from the matched sentence.
-}   
result : Html msg
result = 
       case dictWithMostMatches searchDict dataDict of 
       Nothing ->
           text "Sorry, there is no words in the bag :)"
       Just bestDict ->
           bestDict 
               |> Dict.toList
               |> List.sortBy Tuple.second
               |> List.reverse
               |> List.map Tuple.first
               |> String.join " "
               |> text
           
dictWithMostMatches : Dict String Int -> List (Dict String Int) -> Maybe (Dict String Int)
dictWithMostMatches dict1 listOfDicts =
    List.foldl (score dict1) ( 0, Nothing ) listOfDicts
        |> Tuple.second


score dict1 currentDict ( bestCount, maybeBestDictSoFar ) =
    let
        noOfIdenticalKeys =
            Dict.intersect dict1 currentDict
                |> Dict.size
    in
    case maybeBestDictSoFar of
        Nothing ->
            ( noOfIdenticalKeys, Just currentDict )

        Just bestDictSoFar ->
            if noOfIdenticalKeys > bestCount then
                ( noOfIdenticalKeys, Just currentDict )

            else
                ( bestCount, Just bestDictSoFar )

tokenize : Sentence -> Tokens
tokenize =
    String.filter (\c -> c == ' ' || Char.isAlpha c)
        >> String.toLower
        >> String.words

toHistogram : Tokens -> Histogram
toHistogram =
    List.foldl
        (\key dict ->
            case Dict.get key dict of
                Nothing ->
                    Dict.insert key 1 dict

                Just count ->
                    Dict.insert key (count + 1) dict
        )
        Dict.empty

sentenceHistograms : List Sentence -> List Histogram
sentenceHistograms =
    List.map (tokenize >> toHistogram)

